
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "ILog.h"
#include "OLog.h"
#include "ErrorStrTbl.h"
#include "ErrorStat.h"
#include "gadgets.h"


ErrorStat::ErrorStat(int max) {
	maxErrNo(max);
}

int ErrorStat::maxErrNo() const {
	return theCount-1;
}

void ErrorStat::maxErrNo(int max) {
	max += 2;
	if (theCount < max) {
		stretch(max);
		count(max);
	}
}

void ErrorStat::reset() {
	memset(0);
}

void ErrorStat::record(int errNo) {
	if (errNo < 0 || errNo >= theCount)
		errNo = theCount-1;
	item(errNo)++;
}

void ErrorStat::add(const ErrorStat &s) {
	if (s.count() > count()) {
		stretch(s.count());
		count(s.count());
	}

	for (int e = 0; e < s.count(); ++e)
		item(e) += s[e];
}

ErrorStat &ErrorStat::operator =(const ErrorStat &es) {
	reset();
	count(0);
	add(es);
	return *this;
}

int ErrorStat::totalErrCnt() const {
	int total = 0;
	for (int e = 0; e < theCount; ++e)
		total += item(e);
	return total;
}

OLog &ErrorStat::store(OLog &ol) const {
	return ol << *(const Array<int>*)this;
}

ILog &ErrorStat::load(ILog &il) {
	reset();
	count(0);
	return il >> *(Array<int>*)this;
}

ostream &ErrorStat::print(ostream &os, const ErrorStrTbl &tbl, const String &pfx) const {
	const int total = totalErrCnt();

	os << pfx << "count:\t " << total << endl;

	os << pfx << "tbl:" << endl;
	os << "#errno  count count% explanation" << endl;

	for (int e = 0; e < maxErrNo(); ++e) {
		if (const int ecount = item(e)) {
			os 
				<< setw(6) << e
				<< ' ' << setw(6) << ecount
				<< ' ' << setw(6) << Percent(ecount, total)
				<< " \"" << tbl[e] << '"' << endl;
		}
	}
	os << endl;
	return os;
}
