
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_CONNECTION_HASH_H
#define PG_CONNECTION_HASH_H

#include "Array.h"
#include "Farm.h"
#include "runtime/Connection.h"

// address -> connection map

class NetAddr;
class ConnHashItem;

typedef ConnHashItem **ConnHashPos;

struct ConnHashQueueItem: public QueueItem {
	Connection *conn;

	ConnHashQueueItem(): conn(0) {}
	void reset() { conn = 0; }
};

class ConnHashItem {
	public:
		ConnHashItem();

		void reset();

		bool vacant() const;
		bool matches(const NetAddr &addr) const;

		Queue<ConnHashQueueItem> &queue() { return theQueue; }
		ConnHashItem *collision() { return theCollision; }
		ConnHashPos nextPos() { return &theCollision; }

	protected:
		Queue<ConnHashQueueItem> theQueue;
		ConnHashItem *theCollision;
};

class ConnHash {
	public:
		ConnHash(int ccAddrMax = 0);

		void ccAddrMax(int aMax);

		void add(Connection *conn);
		void del(Connection *conn);
		Connection *delAt(ConnHashPos pos);
		bool find(const NetAddr &addr);
		bool find(const NetAddr &addr, ConnHashPos &pos);

	protected:
		int hash(const NetAddr &addr) const;
		void delAt(ConnHashPos pos, ConnHashQueueItem *qi);

	protected:
		static ObjFarm<ConnHashItem> TheItems;
		static ObjFarm<ConnHashQueueItem> TheQueueItems;
		Array<ConnHashItem*> theHash;
};

#endif
