
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include <limits.h>

#include "String.h"
#include "PglRec.h"
#include "PglBoolSym.h"
#include "PglIntSym.h"
#include "PglNumSym.h"
#include "PglSizeSym.h"
#include "PglBwidthSym.h"
#include "PglDistrSym.h"
#include "PglStringSym.h"
#include "NetPipeSym.h"

#include "gadgets.h"


NetPipeSym::NetPipeSym(): HostsBasedSym("NetPipe") {
	theRec->bAdd("string", "kind", 0);
	theRec->bAdd("time_distr", "packet_delay", 0);
	theRec->bAdd("bwidth", "bandwidth", 0);
	theRec->bAdd("float", "packet_loss", 0);
	theRec->bAdd("int", "queue_packet_size", 0);
	theRec->bAdd("size", "queue_byte_size", 0);
	theRec->bAdd("bool", "outgoing", 0);
}

NetPipeSym::NetPipeSym(const String &aType, PglRec *aRec): HostsBasedSym(aType, aRec) {
}

bool NetPipeSym::isA(const String &type) const {
	return HostsBasedSym::isA(type) || type == "NetPipe";
}

SynSym *NetPipeSym::clone(const String &type) const {
	if (isA(type))
		return new NetPipeSym(this->type(), theRec->clone());
	return HostsBasedSym::clone(type);
}

String NetPipeSym::kind() const {
	return getString("kind");
}

bool NetPipeSym::bandwidthBitsps(int &bitps) const {
	Bwidth byteps = 0;
	if (getBwidth("bandwidth", byteps)) {
		bitps = (int)MiniMax(0.0, 8*byteps, (double)INT_MAX);
		return true;
	}
	return false;
}

RndDistr *NetPipeSym::packetDelay() const {
	return getDistr("packet_delay");
}

bool NetPipeSym::packetLoss(double &plr) const {
	return getDouble("packet_loss", plr);
}

bool NetPipeSym::queuePacketSize(int &size) const {
	return getInt("queue_packet_size", size);
}

bool NetPipeSym::queueByteSize(int &size) const {
	BigSize sz;
	if (getSize("queue_byte_size", sz)) {
		size = sz.byte();
		return true;
	}
	return false;
}

bool NetPipeSym::outgoing() const {
	bool outg;
	if (getBool("outgoing", outg))
		return outg;
	else
		return true;
}

bool NetPipeSym::incoming() const {
	bool outg;
	if (getBool("outgoing", outg))
		return !outg;
	else
		return true;
}
