
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "Clock.h"
#include "LogComment.h"
#include "base/StatIntvlRec.h"
#include "NotifMsg.h"
#include "Notifier.h"

#include "gadgets.h"


Notifier::Notifier(const String &aLabel, const NetAddr &anAddr):
	theLabel(aLabel), theAddr(anAddr) {

	bool ok = true;
	ok = ok && theSock.create(AF_INET, SOCK_DGRAM, 0);
	ok = ok && theSock.connect(theAddr);
	ok = ok && theSock.blocking(false);

	if (!ok) {
		Comment(2) << "warning: cannot connect to " << theAddr << " to send monitoring messages; possible cause: " << error(errno) << endc;
		theSock.close();
	}
}

Notifier::~Notifier() {
	if (active())
		theSock.close();
}

bool Notifier::active() const {
	return theSock >= 0;
}


void Notifier::noteStats(const StatIntvlRec &s, int cat) {
	if (theSock < 0)
		return;
	
	StatusNotifMsg msg(theLabel);

	msg.theStartTime = TheClock.TheStartTime;
	msg.theSndTime = TheClock;
	msg.theRespTime = Time::Msec((int)s.repTime().mean());
	msg.theReqRate = s.reqRate();
	msg.theRepRate = s.repRate();
	msg.theBwidth = Ratio(s.repSize().mean(), s.theDuration.secd());
	msg.theDHR = s.theRealHR.dhr();
	msg.theConnUse = s.theConnUseCnt.mean();
	msg.theErrRatio = s.errRatio();

	msg.theXactTotCnt = TheProgress.xacts();
	msg.theErrTotCnt = TheProgress.errs();
	msg.theSockInstCnt = Socket::Level();
	msg.theCat = cat;

	// write, ignoring errors
	msg.hton();
	theSock.write(&msg, sizeof(msg));
}
