
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "base/UniqId.h"
#include "server/CltGroupRep.h"
#include "server/CltGroupMap.h"

// XXX: we need to grow this map on-demand basis!
CltGroupMap::CltGroupMap(): theIndex(11), theCount(0) {
}

CltGroupMap::~CltGroupMap() {
	while (theIndex.count())
		delete theIndex.pop();
}

CltGroupRep *CltGroupMap::at(int idx) {
	return (0 <= idx && idx < capacity()) ?	theIndex[idx] : 0;
}

CltGroupRep *CltGroupMap::at(const UniqId &group) {
	int idx = -1;
	Assert(find(group, idx));
	return theIndex[idx];
}

CltGroupRep *CltGroupMap::get(const UniqId &group) {
	int idx = -1;
	if (!find(group, idx))
		addAt(idx, group);
	return theIndex[idx];
}

bool CltGroupMap::find(const UniqId &group, int &idx) const {
	if (!capacity())
		return false;

	// start with a hash and then try linear search
	idx = group.hash() % capacity();
	for (int i = capacity(); i; --i) {
		bool res = false;
		if (endSearch(group, idx, res))
			return res;
		idx++;
		idx %= capacity();
	}

	Assert(false); // no empty slots left!
	return false;
}

CltGroupRep *CltGroupMap::addAt(int idx, const UniqId &group) {
	Assert(group);
	Assert(!theIndex[idx]);
	theIndex.put(new CltGroupRep(group, idx), idx);
	theCount++;
	return theIndex[idx];
}

// returns true if there is no reason to search further (match or empty)
bool CltGroupMap::endSearch(const UniqId &group, int idx, bool &res) const {
	if (CltGroupRep *h = theIndex[idx]) {
		if (h->id() == group)
			return res = true;
		return res = false;
	}

	// found empty slot
	res = false;
	return true;
}
