
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "Clock.h"
#include "runtime/ErrorMgr.h"
#include "SrvConnMgr.h"

#include "runtime/polyBcastChannels.h"
#include "polyLogCats.h"
#include "runtime/polyErrors.h"
#include "runtime/globals.h"


SrvConnMgr::SrvConnMgr(User *aUser): theUser(aUser), theAcceptLmt(-1) {
	Assert(theUser);
}

void SrvConnMgr::acceptLmt(int aLmt) {
	theAcceptLmt = aLmt;
}

// return false iff fatal error
bool SrvConnMgr::accept(Socket &s) {
	static const Time acceptTimeLmt = Time::Msec(10);
	const Time startTime = TheClock;
	// note: negative accept limit means no limit
	for (int i = theAcceptLmt; i; --i) {
		bool fatal = false;
		Connection *conn = TheConnFarm.get();
		conn->logCat(lgcSrvSide);
		if (!conn->accept(s, theSockOpt, fatal)) {
			// note: no close() for an un-opened conn
			TheConnFarm.put(conn);
			return !fatal;
		}
		opened(conn);
		conn->theRd.start(this); // wait for data

		// occasionally, check if we have been acceting for too long
		if (i % 10 == 0) {
			Clock::Update(false);
			if (TheClock - startTime > acceptTimeLmt)
				break;
		}
	}
	return true;
}

// this gets called for an idle conn and on the freshly accepted conn
void SrvConnMgr::noteReadReady(int fd) {
	Connection *conn = TheConnIdx[fd];
	Assert(conn);
	TheConnIdx.check(conn);

	conn->theRd.stop(this);
	TheFileScanner->clearTimeout(fd);

	conn->read(); // we do the first read, not a xaction

	if (conn->bad()) { // bad read error
		close(conn);
	} else
	if (conn->exhausted()) { // closed conn with no buffered data
		if (conn->useCnt() > 0)
			conn->closeKind(ConnCloseStat::ckIdleForeign);
		else
			ReportError(errNoHdrClose);
		close(conn);
	} else {
		conn->use();
		theUser->noteConnReady(conn); // more requests or close
	}
}

void SrvConnMgr::putIdle(Connection *conn) {
	Assert(!conn->theRd.theReserv && !conn->theWr.theReserv);
	// there may be more requests waiting
	if (conn->theRdBuf.contSize()) {
		conn->use();
		theUser->noteConnReady(conn);
	} else {
		ConnMgr::putIdle(conn);
	}
}
