
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef XSTD_GADGETS_H
#define XSTD_GADGETS_H

class ostream;

// abs, min, and max
template <class T> inline T Abs(const T &a) { return a > 0 ? a : -a; }
template <class T> inline T Min(const T &a, const T &b) { return (a <= b) ? a : b; }
template <class T> inline T Max(const T &a, const T &b) { return (a >= b) ? a : b; }
template <class T> inline T Min(const T &a, const T &b, const T &c) { return Min(Min(a,b), c); }
template <class T> inline T Max(const T &a, const T &b, const T &c) { return Max(Max(a,b), c); }
template <class T> inline T MiniMax(const T &l, const T &v, const T &h) { return Min(Max(l,v), h); }

// safe division
inline double Ratio(int n, double d) { return d ? (n ? n/d : 0) : -1; }
inline double Ratio(double n, double d) { return d ? n/d : -1; }
inline double Percent(int n, double d) { return Ratio(n, d/100); }
inline double Percent(double n, double d) { return Ratio(n, d/100); }

// new/delete[] style string duplication
extern char *StrDup(const char *str, int len); // dups exactly len chars and appends '\0'
inline char *StrDup(const char *str) { return str ? StrDup(str, strlen(str)) : 0; }

// the greatest common divider
extern int XGCD(int a, int b);

// to avoid problems with signed chars as indices into arrays
inline int xord(char c) { return (int)(unsigned char)c; }

// strNchr(), sort of
const char *StrBoundChr(const char *s, char c, const char *eos);
// finds next white space and skips it
const char *StrBoundAfterSpace(const char *s, const char *eos);

// a better atoi, returns def on error
extern int xatoi(const char *s, int def = 0);

// similar to strto*-like functions
// note: isDbl fails on integers, isNum does not
extern bool isInt(const char *s, int &i, const char **p = 0, int base = 0);
extern bool isDbl(const char *s, double &d, const char **p = 0);
extern bool isNum(const char *s, double &d, const char **p = 0);

// set ``default'' format for a stream and custom precision
void configureStream(ostream &os, int prec);

// returns FD limits
extern int GetCurFD(); // current maximum
extern int GetMaxFD(); // absolute maximum
// attempts to set max number of FDs; returns actual maxFD set or -1 on failure
extern int SetMaxFD(int maxFD);

#endif
